#include <tap.h>
#include "exTap.h"

//----------------------------------------------------------------------------
int exTAP_LED_GetStatus(void);
int exTAP_LED_SetControl(int enable);
int exTAP_LED_GetControl(void);
void exTAP_LED_SendData(byte mask, byte *pData);
byte exTAP_LED_GetMap(char ascii);
int exTAP_LED_SetMap(char ascii, byte sevenseg);
int exTAP_LED_Translate(const char *pIn, byte *pOut);

//---------------------------------------------------------------------
//
// Front Panel LED Patch
//
// this function patches the firmware to allow the front panel display to be manipulated
//

//80006330 27 bd ff d8   addiu $sp, -0x28               # change to: 03 e0 00 08   jr   $ra
//80006334 af b6 00 20   sw    $s6, 0x20($sp)           # change to: 00 00 00 00   nop
type_patch2 led_patch[] = {
    {
        456,
        0x1204,         //firmware revision
        0x800062dc,     //fw_display_func
        8,
        {0x27, 0xbd, 0xff, 0xd8,  0xaf, 0xb6, 0x00, 0x20},
        {0x00, 0x00, 0x00, 0x00,  0x03, 0xe0, 0x00, 0x08},
    },
    {
        0,
    },
};

//pointless...
int exTAP_LED_GetStatus(void)
{
    return 2;
}

int exTAP_LED_SetControl(int enable)
{
    static int id = 0;
    int ret;

    if ((fw_display_string == 0) || (fw_display_func == 0)) {
#ifdef exTAPdebug
        TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
        return EXTAPERR_BAD_FIRMWARE;
    }


    if (enable) {
        ret = exTAP_ApplyPatch2(led_patch, (dword)fw_display_func, &id);
    }
    else {
        ret = exTAP_RemovePatch2(&led_patch[id], (dword)fw_display_func);
        // restore display
        exTAP_LED_SendData(0x1f, fw_display_string);
    }
    return ret;
}

int exTAP_LED_GetControl(void)
{
    int ret;

    //check if firmware is already patched
    ret = exTAP_isPatched2(&led_patch[0], (dword)fw_display_func, NULL);
    if (ret == EXTAPERR_NOT_PATCHED) {
        return 0;
    }
    if (ret == EXTAPERR_ALREADY_PATCHED) {
        return 1;
    }
    return ret;
}


//since the original function has been patched with an 'rts' we need to
//replicate the first 2 instructions, then jump into the firmware function
//at an address offset of 8
static int patched_led_string(void *pFunc)
{
    //cannot use anything that relies on $gp here ..

    __asm__ __volatile__ (
    "or      $08,$00,%0\n"          //or      $t0,$00,pFunc
    "addiu   $08,$08,0x08\n"        //addiu   $t0,$t0,0x08            adjust function entry by 8

    "or      $04,$00,$05\n"         //or      $a0,$00,$a1
    "or      $05,$00,$06\n"         //or      $a1,$00,$a2

    "addiu   $29,$29,0xffd8\n"      //addiu   $sp,$sp,-0x28           first line of pFunc
    "sw      $22,0x0020($29)\n"     //sw      $s6,0x0020($sp)         second line of pFunc

    "jr      $08\n"                 //jr      $t0                     call pFunc(mask, pData)
    :
    : "r"(pFunc));
}

//
// This is a call wrapper for the firmware function displaying
// the specified bytes on the seven segment display of TF5000.
void exTAP_LED_SendData(byte mask, byte *pData)
{
//    exTAP_CallFirmware(patched_led_string, led_patch[firmware_id].info[0].addr, mask, (dword)pData, 0, 0,0,0,0,0,0,0);

    if (fw_display_func == 0) {
#ifdef exTAPdebug
        TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
        return;
    }
    exTAP_CallFirmware(patched_led_string, (dword)fw_display_func, mask, (dword)pData, 0, 0,0,0,0,0,0,0);
}

// This function wraps around the call to a firmware function.
// display elements
//
// control mask
//  bit0 : update first 7-segment from pData[0]
//  bit1 : update second 7-segment from pData[1]
//  bit2 : update third 7-segment from pData[2]
//  bit3 : update fourth 7-segment from pData[3]
//  bit4 : update additional signals from pData[4]
//
// pData should point to an array of 5 bytes
// The first four bytes in the pData array represent the
// seven segment + dot indicators from left to right.
// Bit assignment for seven segment + dor indicators:
//
//     b0
//     --
//  b5|  | b1
//     -- b6
//  b4|  | b2
//     -- o b7 (dot)
//     b3
//
// The fifth byte (pData[4]) represents the state of the
// additional signals (colon, upper dot and LEDs).
// Bit assignment in the fifth byte for additional signals:
//                        b2
//  b5  --     --  b0 --  O  --
//  O  |  |   |  | O |  |   |  |
//  O   --     --  b1 --     --
//  b4 |  |   |  | O |  |   |  |
//      --  o  --  o  --  o  --  o
//
//  b0 - upper dot in the colon
//  b1 - lower dot in the colon
//  b2 - upper dot
//  b4 - green LED
//  b5 - amber LED
//

//these characters are pretty poor...   KM TUVW  ek m uvw

byte charMap[] = {0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x00,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x80,0x40,0x80,0x08,     //spc,-.
                  0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x27, 0x7f,0x6f,0x08,0x08,0x08,0x08,0x08,0x08,     //0-9

                  0x08,0x77,0x7f,0x39,0x3f,0x79,0x71,0x3d, 0x76,0x06,0x1e,0x76,0x38,0x4f,0x37,0x3f,     //,A-O
                  0x73,0xbf,0x77,0x6d,0x78,0x3e,0x3e,0xbe, 0x76,0x6e,0x5b,0x08,0x08,0x08,0x08,0x08,     //P-Z
                  0x08,0xdc,0x7c,0x58,0x5e,0x7b,0x71,0x6f, 0x74,0x04,0x0e,0x76,0x30,0x4f,0x54,0x5c,     //,a-o
                  0x73,0x67,0x50,0x6d,0x78,0x1c,0x1c,0x9c, 0x76,0x6e,0x5b,0x08,0x08,0x08,0x08,0x08,     //p-z

                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,

                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                  0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08, 0x08,0x08,0x08,0x08,0x08,0x08,0x08,0x08,
                 };


byte exTAP_LED_GetMap(char ascii)
{
    return charMap[ascii];

}

int exTAP_LED_SetMap(char ascii, byte sevenseg)
{
    charMap[ascii] = sevenseg;
    return 0;
}

//
// This function translates the specified ASCII string
// to a seven-segment character representation.
int exTAP_LED_Translate(const char *pIn, byte *pOut)
{
    int count = 0;

    while (*pIn != 0) {

        *pOut = charMap[*pIn];

        if ((*(pIn + 1) == '.') || (*(pIn + 1) == ',')) {
            // The next character is a dot or a comma, merge it
            // with the current character if the dot is not
            // already in use.
            if (!(*pOut & 0x80)) {
                *pOut |= 0x80;
                pIn++;
            }
        }

        pIn++;
        pOut++;
        count++;
    }
    return count;
}

//-----------------------------------------------------------------------------
