#include <tap.h>
#include "exTap.h"

//----------------------------------------------------------------------------
int exTAP_MHEG_Disable(int disable);
int exTAP_MHEG_isPatched();
dword exTAP_MHEG_Status();
int exTAP_GetInteractiveStatus(void);
int exTAP_SetInteractiveStatus(int enable);
int exTAP_GetSubtitleStatus(void);

//----------------------------------------------------------------------------
//
// mheg off patch
//
// this patch disables the dsmcc processing in the firmware
// it does not disable the mheg engine from operating if any previously acquired data is present
// changing channels will flush the dsmcc file caches
//
//patch out dsmcc handler
//801aacfc 27 bd ff e0   addiu $sp, -0x20               03 e0 00 08    jr   $ra
//801aad00 af b6 00 18   sw    $s6, 0x18($sp)           00 00 00 00    nop

type_patch2 mheg_off_patch[] = {
    {
        456,               //system id
        -1,                //firmware revision
        -1,                //fw_process_dsmcc; -1 = user supplied
        8,
        {0x27, 0xbd, 0xff, 0xc8,  0xaf, 0xb3, 0x00, 0x24},      //12.88
        {0x03, 0xe0, 0x00, 0x08,  0x00, 0x00, 0x00, 0x00},
    },
    {
        456,               //system id
        -1,                //firmware revision; -1=any
        -1,                //fw_process_dsmcc; -1 = user supplied
        8,
        {0x27, 0xbd, 0xff, 0x78,  0xaf, 0xb6, 0x00, 0x80},      //13.40
        {0x03, 0xe0, 0x00, 0x08,  0x00, 0x00, 0x00, 0x00},
    },
    {
        0,
    },
};

int exTAP_MHEG_Disable(int disable)
{
    static int id = 0;

    if (fw_process_dsmcc == 0) {
#ifdef exTAPdebug
        TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
        return EXTAPERR_BAD_FIRMWARE;
    }
    if (disable) {
        return exTAP_ApplyPatch2(&mheg_off_patch[0], (dword)fw_process_dsmcc, &id);
    }
    else {
        return exTAP_RemovePatch2(&mheg_off_patch[id], (dword)fw_process_dsmcc);
    }
}

int exTAP_MHEG_isPatched()
{
    int ret;

    ret = exTAP_isPatched2(&mheg_off_patch[0], (dword)fw_process_dsmcc, NULL);
    if (ret == EXTAPERR_NOT_PATCHED) return 0;
    if (ret == EXTAPERR_ALREADY_PATCHED) return 1;
    return ret;
}

dword exTAP_MHEG_Status()
{
    dword v0,v1;
    dword register_group;

    if (fw_mheg_entry == 0) {
#ifdef exTAPdebug
        TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
        return EXTAPERR_BAD_FIRMWARE;
    }

    register_group = 0;
    v0 = *fw_mheg_entry;
    if (v0) {
        v1 = *(dword *)(v0 + 0x8);
        if (v1) {
            register_group = *(dword *)(v1 + 0x34);
        }
    }

    //see  dtt mheg-5 spec. v1.06, section 3.6
    //if register_group == 0 then no mheg is running
    //if register_group == 3 then mheg is running, has claimed <text><red><green><yellow><blue>
    //if register_group == 5 then mheg is running, has claimed group3 + <arrow keys>
    //if register_group == 4 then mheg is running, has claimed group3 + group5 + <number keys>
    return register_group;
}

//
//this will return the current interactive setting, 0=off, 1=on
//
int exTAP_GetInteractiveStatus(void)
{
    type_etc_structure *etc;

    if (fw_etc_structure == 0) {
#ifdef exTAPdebug
        TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
        return EXTAPERR_BAD_FIRMWARE;
    }

    etc = (type_etc_structure *)*(dword *)fw_etc_structure;
    if (etc->byte_1c & 0x20) {
        return 0;   //disabled
    }
    return 1;   //enabled
}

//
//this will enable/disable interactive immediately, and permanently (i.e. updates the flash)
//it acts the same as using the menus
//
int exTAP_SetInteractiveStatus(int enable)
{
    type_etc_structure *etc;

    if ((fw_etc_structure == 0) || (fw_mheg_disable == 0)) {
#ifdef exTAPdebug
        TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
        return EXTAPERR_BAD_FIRMWARE;
    }

    etc = (type_etc_structure *)*(dword *)fw_etc_structure;
    if (enable) {
        if (etc->byte_1c & 0x20) {
            etc->byte_1c ^= 0x20;
            TAP_WriteSystemVar();
//          TAP_Print("\nInteractive On");
        }
    }
    else {
        if ((etc->byte_1c & 0x20) == 0) {
            etc->byte_1c ^= 0x20;
            TAP_WriteSystemVar();
            exTAP_CallFirmware((void *)fw_mheg_disable, 0,0,0,0,0,0,0,0,0,0,0);
//          TAP_Print("\nInteractive Off");
        }
    }
    return EXTAPERR_NONE;
}

//
//this will return the current subtitle setting, 0=off, 1=on
//
int exTAP_GetSubtitleStatus(void)
{
    type_etc_structure *etc;

    if (fw_etc_structure == 0) {
#ifdef exTAPdebug
        TAP_Print("\n%s(), Unsupported firmware: sysid:%d, version:%04x", __func__, fw_sysid, _appl_version);
#endif
        return EXTAPERR_BAD_FIRMWARE;
    }

    etc = (type_etc_structure *)*(dword *)fw_etc_structure;
    if (etc->byte_0d & 0x10) {
        return 0;   //disabled
    }
    return 1;   //enabled
}

//---------------------------------------------------------------------
